Package: aggregateBioVar
Type: Package
Title: Differential Gene Expression Analysis for Multi-subject
        scRNA-seq
Version: 1.21.0
Authors@R: c(
    person(given = "Jason", family = "Ratcliff",
        email = "jason-ratcliff@uiowa.edu", role = c("aut", "cre"),
        comment = c(ORCID = "0000-0001-7079-334X")),
    person(given = "Andrew", family = "Thurman", 
        email = "andrew.thurman2@gmail.com", role = "aut"),
    person(given = "Michael", family = "Chimenti", 
        email = "michael-chimenti@uiowa.edu", role = "ctb"),
    person(given = "Alejandro", family = "Pezzulo", 
        email = "alejandro-pezzulo@uiowa.edu", role = "ctb")
    )
URL: https://github.com/jasonratcliff/aggregateBioVar
BugReports: https://github.com/jasonratcliff/aggregateBioVar/issues
Description: For single cell RNA-seq data collected from more than 
    one subject (e.g. biological sample or technical replicates),
    this package contains tools to summarize single cell gene expression
    profiles at the level of subject. A SingleCellExperiment object is taken as
    input and converted to a list of SummarizedExperiment objects, where each
    list element corresponds to an assigned cell type.
    The SummarizedExperiment objects contain aggregate gene-by-subject
    count matrices and inter-subject column metadata for individual subjects
    that can be processed using downstream bulk RNA-seq tools.
License: GPL-3
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.1.1
biocViews: Software, SingleCell, RNASeq, Transcriptomics,
        Transcription, GeneExpression, DifferentialExpression
Depends: R (>= 4.0)
Imports: stats, methods, S4Vectors, SummarizedExperiment,
        SingleCellExperiment, Matrix, tibble, rlang
Suggests: BiocStyle, magick, knitr, rmarkdown, testthat, BiocGenerics,
        DESeq2, magrittr, dplyr, ggplot2, cowplot, ggtext,
        RColorBrewer, pheatmap, viridis
git_url: https://git.bioconductor.org/packages/aggregateBioVar
git_branch: devel
git_last_commit: b0e2560
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
NeedsCompilation: no
Packaged: 2025-11-03 21:09:09 UTC; biocbuild
Author: Jason Ratcliff [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7079-334X>),
  Andrew Thurman [aut],
  Michael Chimenti [ctb],
  Alejandro Pezzulo [ctb]
Maintainer: Jason Ratcliff <jason-ratcliff@uiowa.edu>

# Generated by roxygen2: do not edit by hand

export(aggregateBioVar)
export(countsByCell)
export(countsBySubject)
export(scSubjects)
export(subjectMetaData)
export(summarizedCounts)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(rlang,"!!")
importFrom(rlang,":=")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# aggregateBioVar

<!-- badges: start -->

<!-- badges: end -->

Single cell RNA sequencing (scRNA-seq) studies allow gene expression
quantification at the level of individual cells. These studies introduce
multiple layers of biological complexity, including variations in gene
expression between cell states within a sample (*e.g.*, T cells versus
macrophages), between samples within a population (*e.g.*, biological or
technical replicates), and between populations (*e.g.*, healthy versus
diseased individuals). Many early scRNA-seq studies involved analysis of
gene expression within cells from a *single sample*. For single cell
RNA-seq data collected from more than one subject, `aggregateBioVar`
provides tools to summarize summarize single cell gene expression
profiles at the level of *samples* (*i.e.*, subjects) or *populations*.
Given an input
[`SingleCellExperiment`](https://bioconductor.org/packages/release/bioc/vignettes/SingleCellExperiment/inst/doc/intro.html)
object (Amezquita et al. [2020](#ref-SingleCellExperiment2020)) with
pre-defined cell states, `aggregateBioVar()` stratifies data as a list
of
[`SummarizedExperiment`](https://bioconductor.org/packages/release/bioc/vignettes/SummarizedExperiment/inst/doc/SummarizedExperiment.html)
objects (Morgan et al. [2020](#ref-R-SummarizedExperiment)). For each
cell type, gene counts are aggregated by subject into a
**gene-by-subject** count matrix, and column metadata are summarized to
retain *inter-subject* variation for downstream analysis with bulk
RNA-seq tools.

## Installation

<!-- # TODO Replace with BiocManager::install() -->

<!-- You can install the released version of aggregateBioVar from -->

<!-- [CRAN](https://CRAN.R-project.org) with: -->

<!-- ``` r -->

<!-- install.packages("aggregateBioVar") -->

<!-- ``` -->

Install the development version of `aggregateBioVar` from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("jasonratcliff/aggregateBioVar", build_vignettes=TRUE)
```

## Multi-subject scRNA-seq

``` r
library(aggregateBioVar)

# Bioconductor Packages
library(SummarizedExperiment, quietly = TRUE)
library(SingleCellExperiment, quietly = TRUE)
library(DESeq2, quietly = TRUE)

# Data analysis and visualization
library(dplyr, quietly = TRUE)
library(magrittr, quietly = TRUE)
library(ggplot2, quietly = TRUE)
library(ggtext, quietly = TRUE)
```

To illustrate the utility of *biological replication* for scRNA-seq
sequencing experiments, consider a `SingleCellExperiment` object with
scRNA-seq data from 7 subjects (SCF1, SCF2, SCF3, SWT1, SWT2, SWT3,
SWT4) in the context of a cystic fibrosis phenotype. Samples were
collected from small airway epithelium of newborn *Sus scrofa* with
genotypes from wild type (**CFTR+/+**, n=4) and *CFTR*-knockout
(**CFTR-/-**, n=3) individuals. Note the dimensions of this object, with
1339 genes from 2722 cells:

``` r
small_airway
#> class: SingleCellExperiment 
#> dim: 1339 2722 
#> metadata(0):
#> assays(1): counts
#> rownames(1339): MPC1 PRKN ... OTOP1 UNC80
#> rowData names(0):
#> colnames(2722): SWT1_AAAGAACAGACATAAC SWT1_AAAGGATTCTCCGAAA ...
#>   SCF3_TTTGGAGGTAAGGCTG SCF3_TTTGGTTCAATAGTAG
#> colData names(6): orig.ident nCount_RNA ... Region celltype
#> reducedDimNames(0):
#> altExpNames(0):
```

The primary function `aggregateBioVar()` takes a `SingleCellExperiment`
object with column metadata variables indicating subject identity
(*e.g.*, biological sample; `subjectVar`) and assigned cell type
(`cellVar`). The column metadata of a `SingleCellExperiment` object can
be obtained by `SummarizedExperiment::colData()`. Here, the metadata
variable `orig.ident` indicates the biological sample identifier and
`celltype` the inferred cell type.

``` r
# Perform aggregation of counts and metadata by subject and cell type.
aggregate_counts <-
    aggregateBioVar(
        scExp = small_airway,
        subjectVar = "orig.ident", cellVar = "celltype"
    )
#> Coercing metadata variable to character: celltype
```

Each element of the returned list contains a `SummarizedExperiment`
object with aggregated counts from cells in the assigned cell type
(indicated by `cellVar`).

``` r
aggregate_counts
#> $AllCells
#> class: SummarizedExperiment 
#> dim: 1339 7 
#> metadata(0):
#> assays(1): counts
#> rownames(1339): MPC1 PRKN ... OTOP1 UNC80
#> rowData names(0):
#> colnames(7): SWT1 SWT2 ... SWT4 SCF3
#> colData names(3): orig.ident Genotype Region
#> 
#> $`Immune cell`
#> class: SummarizedExperiment 
#> dim: 1339 7 
#> metadata(0):
#> assays(1): counts
#> rownames(1339): MPC1 PRKN ... OTOP1 UNC80
#> rowData names(0):
#> colnames(7): SWT1 SWT2 ... SWT4 SCF3
#> colData names(4): orig.ident Genotype Region celltype
#> 
#> $`Secretory cell`
#> class: SummarizedExperiment 
#> dim: 1339 7 
#> metadata(0):
#> assays(1): counts
#> rownames(1339): MPC1 PRKN ... OTOP1 UNC80
#> rowData names(0):
#> colnames(7): SWT1 SWT2 ... SWT4 SCF3
#> colData names(4): orig.ident Genotype Region celltype
#> 
#> $`Endothelial cell`
#> class: SummarizedExperiment 
#> dim: 1339 7 
#> metadata(0):
#> assays(1): counts
#> rownames(1339): MPC1 PRKN ... OTOP1 UNC80
#> rowData names(0):
#> colnames(7): SWT1 SWT2 ... SWT4 SCF3
#> colData names(4): orig.ident Genotype Region celltype
```

For each cell type subset, within-subject gene counts are aggregated and
column metadata are summarized to exclude variables with *intercellular*
variation. This effectively retains subject metadata and can be used for
downstream analysis with bulk RNA-seq tools. After aggregation, the
number of columns in the `SingleCellExperiment` object matches the
number of unique values in the subject metadata variable indicated by
`subjectVar`.

``` r
assay(aggregate_counts$`Immune cell`, "counts")
#> DataFrame with 1339 rows and 7 columns
#>              SWT1      SWT2      SWT3      SCF1      SCF2      SWT4      SCF3
#>         <numeric> <numeric> <numeric> <numeric> <numeric> <numeric> <numeric>
#> MPC1          257       209        56       315        33        93        42
#> PRKN            1         0         0         0         0         2         0
#> SLC22A3         0         0         0         1         0         0         0
#> CNKSR3          3         4         5         8         1         1         6
#> UTRN          166       155        32       233        27       116        20
#> ...           ...       ...       ...       ...       ...       ...       ...
#> CNNM1           0         0         0         0         0         0         0
#> GABRQ           0         0         0         0         0         0         0
#> GIF             0         0         0         0         0         0         0
#> OTOP1           0         0         0         0         0         0         0
#> UNC80           0         0         0         0         0         0         0
```

``` r
colData(aggregate_counts$`Immune cell`)
#> DataFrame with 7 rows and 4 columns
#>       orig.ident    Genotype      Region    celltype
#>      <character> <character> <character>    <factor>
#> SWT1        SWT1          WT       Small Immune cell
#> SWT2        SWT2          WT       Small Immune cell
#> SWT3        SWT3          WT       Small Immune cell
#> SCF1        SCF1      CFTRKO       Small Immune cell
#> SCF2        SCF2      CFTRKO       Small Immune cell
#> SWT4        SWT4          WT       Small Immune cell
#> SCF3        SCF3      CFTRKO       Small Immune cell
```

### Differential Gene Expression

The aggregate gene-by-subject matrix and subject metadata can be used as
inputs for bulk RNA-seq tools to investigate gene expression. Here, an
example is provided using
[`DESeq2`](https://bioconductor.org/packages/devel/bioc/vignettes/DESeq2/inst/doc/DESeq2.html)
(Love, Huber, and Anders [2014](#ref-DESeq2)). A `DESeqDataSet` can be
constructed from the aggregate gene-by-subject count matrix and
summarized column metadata.

``` r
subj_dds_dataset <-
    DESeqDataSetFromMatrix(
        countData = assay(aggregate_counts$`Secretory cell`, "counts"),
        colData = colData(aggregate_counts$`Secretory cell`),
        design = ~ Genotype
    )
#> converting counts to integer mode
#> Warning in DESeqDataSet(se, design = design, ignoreRank): some variables in
#> design formula are characters, converting to factors

subj_dds <- DESeq(subj_dds_dataset)
#> estimating size factors
#> estimating dispersions
#> gene-wise dispersion estimates
#> mean-dispersion relationship
#> final dispersion estimates
#> fitting model and testing

subj_dds_results <-
    results(subj_dds, contrast = c("Genotype", "WT", "CFTRKO"))
```

Add negative log<sub>10</sub> adjusted P-values, then plot against
log<sub>2</sub> fold change. Genes with adjusted P-values \< 0.05 and
fold-change absolute values \> 1.0 are highlighted in red and labeled by
feature.

``` r
subj_dds_transf <- as.data.frame(subj_dds_results) %>%
    bind_cols(feature = rownames(subj_dds_results)) %>%
    mutate(log_padj = - log(.data$padj, base = 10))

ggplot(data = subj_dds_transf) +
    geom_point(aes(x = log2FoldChange, y = log_padj), na.rm = TRUE) +
    geom_point(
        data = filter(
            .data = subj_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange, y = log_padj), color = "red"
    ) +
    geom_label(
        data = filter(
            .data = subj_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange, y = log_padj + 0.4, label = feature)
    ) +
    theme_classic() +
    labs(
        x = "log<sub>2</sub> (fold change)",
        y = "-log<sub>10</sub> (p<sub>adj</sub>)"
    ) +
    theme(
        axis.title.x = element_markdown(),
        axis.title.y = element_markdown())
```

<img src="man/figures/README-volcanoPlot-1.png" width="75%" />

## Vignettes

For a detailed workflow and description of package components, see the
package vignette:

``` r
vignette("multi-subject-scRNA-seq", package = "aggregateBioVar")
```

## References

<div id="refs">

<div id="ref-SingleCellExperiment2020">

Amezquita, Robert, Aaron Lun, Etienne Becht, Vince Carey, Lindsay Carpp,
Ludwig Geistlinger, Federico Marini, et al. 2020. “Orchestrating
Single-Cell Analysis with Bioconductor.” *Nature Methods* 17: 137–45.
<https://www.nature.com/articles/s41592-019-0654-x>.

</div>

<div id="ref-R-magrittr">

Bache, Stefan Milton, and Hadley Wickham. 2014. *magrittr: A
Forward-Pipe Operator for R*.
<https://CRAN.R-project.org/package=magrittr>.

</div>

<div id="ref-DESeq2">

Love, Michael I., Wolfgang Huber, and Simon Anders. 2014. “Moderated
Estimation of Fold Change and Dispersion for RNA-seq Data with DESeq2.”
*Genome Biology* 15 (12): 550.
<https://doi.org/10.1186/s13059-014-0550-8>.

</div>

<div id="ref-R-SummarizedExperiment">

Morgan, Martin, Valerie Obenchain, Jim Hester, and Hervé Pagès. 2020.
*SummarizedExperiment: SummarizedExperiment Container*.

</div>

<div id="ref-R-dplyr">

Wickham, Hadley, Romain François, Lionel Henry, and Kirill Müller. 2020.
*dplyr: A Grammar of Data Manipulation*.
<https://CRAN.R-project.org/package=dplyr>.

</div>

<div id="ref-R-cowplot">

Wilke, Claus O. 2019. *cowplot: Streamlined Plot Theme and Plot
Annotations for ’ggplot2’*.
<https://CRAN.R-project.org/package=cowplot>.

</div>

<div id="ref-R-ggtext">

———. 2020. *ggtext: Improved Text Rendering Support for ’ggplot2’*.
<https://CRAN.R-project.org/package=ggtext>.

</div>

</div>

## Count Aggregation by Cell Type ----

#' Within cell type gene-by-subject matrices
#'
#' Given a vector of unique cell types, calculate a gene-by-subject matrix and
#' inter-subject metadata for a differential expression design matrix.
#'
#' @inheritParams aggregateBioVar
#' @export
#'
#' @return List of gene-by-subject and design matrices for each cell type.
#' @seealso \code{\link{summarizedCounts}} for aggregate counts and metadata
#'   \linkS4class{SummarizedExperiment} object.
#'
#' @examples
#' ## Return list of `SummarizedExperiments` with gene-by-subject count matrices
#' ## and subject metadata for each unique `SingleCellExperiment` cell type.
#' countsByCell(
#'     scExp=small_airway,
#'     subjectVar="orig.ident", cellVar="celltype"
#' )
#'
countsByCell <- function(scExp, subjectVar, cellVar) {

    cellTypes <-
        scSubjects(scExp, subjects=subjectVar, cellTypes=cellVar)[["cellTypes"]]
    cellCountList <-
        lapply(cellTypes, function(cellType) {

            ## Subset counts and metadata by cell type into new SCE object.
            cellIndex <- which(scExp[[cellVar]] == cellType)
            cellCounts <- SingleCellExperiment::counts(scExp)[, cellIndex]
            cellMetaData <- SummarizedExperiment::colData(scExp)[cellIndex, ]
            cellAggregate <-
                SingleCellExperiment::SingleCellExperiment(
                    assays=list("counts"=cellCounts),
                    colData=cellMetaData
                )

            ## Aggregate within-cell type gene counts and metadata by subject.
            summarizedCounts(scExp=cellAggregate, subjectVar=subjectVar)
            })

    cellCountList <- stats::setNames(object=cellCountList, nm=cellTypes)
    return(cellCountList)
}

#' Aggregate subject-level biological variation
#'
#' Given an input gene-by-cell count matrix from a
#' \linkS4class{SingleCellExperiment} object, sum within-subject gene counts
#' into an aggregate gene-by-subject count matrix. Column metadata accessed
#' by \link[SummarizedExperiment:SummarizedExperiment-class]{colData}
#' are collated by \link{subjectMetaData} to remove variables with inter-cell
#' intrasubject variation, effectively retaining between-subject variation.
#' The summary operations are performed across all cell types and within each
#' cell type. A list of \linkS4class{SummarizedExperiment} objects is returned
#' each with aggregate gene-by-subject count matrix and inter-subject metadata.
#'
#' @param scExp \linkS4class{SingleCellExperiment} object containing
#'   (at minimum) gene counts and column metadata describing sample identifiers
#'   and cell types.
#' @param subjectVar Metadata column name assigning biological sample
#'   identity to aggregate within-subject feature counts.
#' @param cellVar Metadata column name assigning cell type. Used for
#'   aggregating gene-by-subject count matrices by cell type.
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @export
#'
#' @return List of \linkS4class{SummarizedExperiment} objects with
#'   gene-by-subject count matrices and variable inter-subject column
#'   metadata across and within cell types.
#'
#' @examples
#' ## Aggregate gene-by-subject count matrix and inter-subject metadata
#' aggregateBioVar(
#'     scExp=small_airway,
#'     subjectVar="orig.ident", cellVar="celltype"
#' )
#'
aggregateBioVar <- function(scExp, subjectVar, cellVar) {

    ## Cast single cell input as `SingleCellExperiment` class
    if (!methods::is(scExp, "SingleCellExperiment")) {
        stop(paste0("`scExp` must be an object of SingleCellExperiment class:",
                    "\n\nhttps://bioconductor.org/packages/release/",
                    "bioc/html/SingleCellExperiment.html"))
    }

    ## Notify subsetting for multiple `SingleCellExperiment` assay slots.
    if (length(SummarizedExperiment::assays(scExp)) > 1) {
        assayName <- SummarizedExperiment::assayNames(scExp)[1]
        message(paste("SingleCellExperiment counts from assay:", assayName))
        scExp <- SingleCellExperiment::SingleCellExperiment(
            assay=list("counts"=SummarizedExperiment::assay(
                scExp, assayName
            )),
            colData=SummarizedExperiment::colData(scExp)
        )
    }

    ## Notify metadata variable class to character vector coercion.
    sceClasses <- list(
        "subjectVar"=class(scExp[[subjectVar]]),
        "cellVar"=class(scExp[[cellVar]])
    )
    if (TRUE %in% (!sceClasses %in% "character")) {
        classVars <- names(sceClasses)[!sceClasses %in% "character"]
        varMsg <- vapply(
            X=classVars, FUN.VALUE=character(1),
            FUN=function(var) eval(rlang::sym(var))
        )
        message(
            "Coercing metadata variable to character: ",
            paste(varMsg, collapse=" ")
        )
    }

    ## Combine summarized counts across all cells with aggregation by cell type.
    summarizedAll <- summarizedCounts(scExp=scExp, subjectVar=subjectVar)
    summarizedByCell <-
        countsByCell(scExp=scExp, subjectVar=subjectVar, cellVar=cellVar)
    aggregateSingleCell <- c(list(AllCells=summarizedAll), summarizedByCell)
    return(aggregateSingleCell)
}

test_that("Cell Type Wrapper", {
    cellSummaries <- countsByCell(
        scExp = small_airway,
        subjectVar = "orig.ident", cellVar = "celltype"
    )
    expect_equivalent(
        names(cellSummaries),
        unique(as.character(small_airway$celltype))
    )
    expect_identical(
        unique(vapply(cellSummaries, class, character(1))),
        "SummarizedExperiment"
    )
})

test_that("Cell Type Wrapper", {
    library(SummarizedExperiment)
    library(SingleCellExperiment)

    ## Error for non-`SingleCellExperiment` object input for `scExp`.
    seWarning <-
        SummarizedExperiment(
            assays = list("counts" = assay(small_airway, "counts")),
            colData = colData(small_airway)
        )
    expect_error(
        aggregateBioVar(
            scExp = seWarning, subjectVar = "orig.ident", cellVar = "celltype"
        )
    )

    ## Message to subset first assay slot if multiple slots are present.
    scExpMultiAssay <- SingleCellExperiment(
        assay = list(
            "RNA" = assay(small_airway, "counts"),
            "counts" = assay(small_airway, "counts")
        ),
        colData = colData(small_airway)
    )
    expect_message(
        aggregateCounts <- aggregateBioVar(
            scExp = scExpMultiAssay,
            subjectVar = "orig.ident", cellVar = "celltype"
        ), regexp = "counts from assay: RNA"
    )

    ## Message for character vector coercion of metadata variables.
    expect_message(
        aggregateBioVar(
            scExp = small_airway,
            subjectVar = "orig.ident", cellVar = "celltype"
        ), regexp = "Coercing metadata variable to character: celltype"
    )

    ## Expected return list output with cell type names.
    expect_is(aggregateCounts, "list")
    expect_equivalent(
        c("AllCells", unique(as.character(small_airway$celltype))),
        names(aggregateCounts)
    )
})

## ----vignette, include = FALSE------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
knitr::opts_knit$set(
    eval.after = "fig.cap"
)

## ----setup, message=FALSE-----------------------------------------------------
# For analysis of scRNAseq data
library(aggregateBioVar)
library(SummarizedExperiment, quietly = TRUE)
library(SingleCellExperiment, quietly = TRUE)
library(DESeq2, quietly = TRUE)

# For data transformation and visualization
library(magrittr, quietly = TRUE)
library(dplyr, quietly = TRUE)
library(ggplot2, quietly = TRUE)
library(cowplot, quietly = TRUE)
library(ggtext, quietly = TRUE)

## ----links, echo=FALSE--------------------------------------------------------
link_sce <-
    paste0(
        "https://bioconductor.org/packages/release/bioc/",
        "vignettes/SingleCellExperiment/inst/doc/intro.html"
    )
link_se <-
    paste0(
        "https://bioconductor.org/packages/release/bioc/",
        "vignettes/SummarizedExperiment/inst/doc/SummarizedExperiment.html"
    )
link_ncbi <-
    paste0(
        "https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/",
        "wwwtax.cgi?mode=Info&id=9823"
    )
link_DESeq2 <-
    paste0(
        "https://bioconductor.org/packages/devel/bioc/",
        "vignettes/DESeq2/inst/doc/DESeq2.html"
    )

## ----captions, echo=FALSE, message=FALSE--------------------------------------
cap_metadata <-
    SummarizedExperiment::colData(x = small_airway) %>% tibble::as_tibble() %>%
    dplyr::select("orig.ident", "Genotype") %>%
    dplyr::distinct() %>% dplyr::arrange(.data$orig.ident)

# Figure 1
cap_pheatmanp_cell <-
    paste0(
        "Gene-by-cell Count Matrix. ",
        "Heatmap of *secretory cell* gene expression with log~2~ counts ",
        "per million cells. Includes ",
        length(grep("Secretory cell", small_airway$celltype)),
        " cells from ", length(unique(small_airway$orig.ident)), " subjects",
        " with genotypes `WT` (n=", length(grep("WT", cap_metadata$Genotype)),
        ", cells=",
        length(
            intersect(
                grep("WT", small_airway$Genotype),
                grep("Secretory cell", small_airway$celltype)
            )
        ),
        ") and `CFTRKO` (n=", length(grep("CFTRKO", cap_metadata$Genotype)),
        ", cells=",
        length(
            intersect(
                grep("CFTRKO", small_airway$Genotype),
                grep("Secretory cell", small_airway$celltype)
            )
        ),
        ")."
    )

# Figure 2
cap_pheatmanp_subj <-
    paste0(
        "Gene-by-subject Count Matrix. ",
        "Heatmap of gene counts aggregated by ",
        "subject with `aggregateBioVar()`.",
        "The `SummarizedExperiment` object with the *Secretory cells* subset ",
        "contains gene counts summed by subject. Aggregate gene-by-subject ",
        "counts are used as input for bulk RNA-seq tools."
    )

# Figure 3
cap_volcano <-
    paste0(
        "Differential expression analysis of scRNA-seq data. ",
        "Comparison of differential expression in ",
        "secretory cells from small airway epithelium ",
        "without aggregation (A) and after aggregation of gene counts ",
        "by subject (biological replicates; B). ",
        "Genes with an absolute log~2~ fold change greater than 1 and an ",
        "adjusted P value less than 0.05 are highlighted in red. Aggregation ",
        "of counts by subject reduced the number of differentially expressed ",
        "genes to CD36 and CFTR for the secretory cell subset."
    )

# Figure 4
cap_counts <-
    paste0(
        "Normalized within-subject gene counts. ",
        "Gene counts aggregated by subject for significantly differentially ",
        "expressed genes from the secretory cell subset."
    )

## ----singleCellExperiment-----------------------------------------------------
small_airway

## ----assays-------------------------------------------------------------------
assays(small_airway)

# Dimensions of gene-by-cell count matrix
dim(counts(small_airway))

# Access dgCMatrix with gene counts
counts(small_airway)[1:5, 1:30]

## ----subjects, echo=FALSE-----------------------------------------------------
sbj_cf <-
    grep(pattern = "CF", x = unique(small_airway$orig.ident), value = TRUE)
sbj_wt <-
    grep(pattern = "WT", x = unique(small_airway$orig.ident), value = TRUE)

## ----metadata-----------------------------------------------------------------
# Subject values
table(small_airway$orig.ident)

# Cell type values
table(small_airway$celltype)

# Subject genotype
table(small_airway$Genotype)

## ----columnData---------------------------------------------------------------
colData(small_airway)

## ----countsBySubject----------------------------------------------------------
countsBySubject(scExp = small_airway, subjectVar = "orig.ident")

## ----subjectMetaData----------------------------------------------------------
subjectMetaData(scExp = small_airway, subjectVar = "orig.ident")

## ----summarizedCounts---------------------------------------------------------
summarizedCounts(scExp = small_airway, subjectVar = "orig.ident")

## ----aggregateBioVar----------------------------------------------------------
aggregateBioVar(scExp = small_airway,
                subjectVar = "orig.ident", cellVar = "celltype")

## ----DESeq2Example------------------------------------------------------------
# Perform aggregation of counts and metadata by subject and cell type.
aggregate_counts <-
    aggregateBioVar(
        scExp = small_airway,
        subjectVar = "orig.ident", cellVar = "celltype"
    )

## ----pheatmapFxn--------------------------------------------------------------
#' Single-cell Counts `pheatmap`
#'
#' @param sumExp `SummarizedExperiment` or `SingleCellExperiment` object
#'   with individual cell or aggregate counts by-subject.
#' @param logSample Subset of log2 values to include for clustering.
#' @param ... Forwarding arguments to pheatmap
#' @inheritParams aggregateBioVar
#'
scPHeatmap <- function(sumExp, subjectVar, gtVar, logSample = 1:100, ...) {
    orderSumExp <- sumExp[, order(sumExp[[subjectVar]])]
    sumExpCounts <- as.matrix(
        SummarizedExperiment::assay(orderSumExp, "counts")
    )
    logcpm <- log2(
        1e6*t(t(sumExpCounts) / colSums(sumExpCounts)) + 1
    )
    annotations <- data.frame(
        Genotype = orderSumExp[[gtVar]],
        Subject = orderSumExp[[subjectVar]]
    )
    rownames(annotations) <- colnames(orderSumExp)

    singleCellpHeatmap <- pheatmap::pheatmap(
        mat = logcpm[logSample, ], annotation_col = annotations,
        cluster_cols = FALSE, show_rownames = FALSE, show_colnames = FALSE,
        scale = "none", ...
    )
    return(singleCellpHeatmap)
}


## ----pheatmapCells, fig.cap=cap_pheatmanp_cell--------------------------------
# Subset `SingleCellExperiment` secretory cells.
sumExp <- small_airway[, small_airway$celltype == "Secretory cell"]

# List of annotation color specifications for pheatmap.
ann_colors <- list(
    Genotype = c(CFTRKO = "red", WT = "black"),
    Subject = c(RColorBrewer::brewer.pal(7, "Accent"))
)
ann_names <- unique(sumExp[["orig.ident"]])
names(ann_colors$Subject) <- ann_names[order(ann_names)]

# Heatmap of log2 expression across all cells.
scPHeatmap(
    sumExp = sumExp, logSample = 1:100,
    subjectVar = "orig.ident", gtVar = "Genotype",
    color = viridis::viridis(75), annotation_colors = ann_colors,
    treeheight_row = 0, treeheight_col = 0
)

## ----pheatmapSubject, fig.cap=cap_pheatmanp_subj------------------------------
# List of `SummarizedExperiment` objects with aggregate subject counts.
scExp <-
    aggregateBioVar(
        scExp = small_airway,
        subjectVar = "orig.ident", cellVar = "celltype"
    )

# Heatmap of log2 expression from aggregate gene-by-subject count matrix.
scPHeatmap(
    sumExp = aggregate_counts$`Secretory cell`, logSample = 1:100,
    subjectVar = "orig.ident", gtVar = "Genotype",
    color = viridis::viridis(75), annotation_colors = ann_colors,
    treeheight_row = 0, treeheight_col = 0
)

## ----DESeq2Aggregate----------------------------------------------------------
subj_dds_dataset <-
    DESeqDataSetFromMatrix(
        countData = assay(aggregate_counts$`Secretory cell`, "counts"),
        colData = colData(aggregate_counts$`Secretory cell`),
        design = ~ Genotype
    )

subj_dds <- DESeq(subj_dds_dataset)

subj_dds_results <-
    results(subj_dds, contrast = c("Genotype", "WT", "CFTRKO"))

## ----DESeq2Cells--------------------------------------------------------------
cells_secretory <-
    small_airway[, which(
        as.character(small_airway$celltype) == "Secretory cell")]
cells_secretory$Genotype <- as.factor(cells_secretory$Genotype)

cell_dds_dataset <-
    DESeqDataSetFromMatrix(
        countData = assay(cells_secretory, "counts"),
        colData = colData(cells_secretory),
        design = ~ Genotype
    )

cell_dds <- DESeq(cell_dds_dataset)

cell_dds_results <-
    results(cell_dds, contrast = c("Genotype", "WT", "CFTRKO"))

## ----logPadj, message=FALSE---------------------------------------------------
subj_dds_transf <- as.data.frame(subj_dds_results) %>%
    bind_cols(feature = rownames(subj_dds_results)) %>%
    mutate(log_padj = - log(.data$padj, base = 10))

cell_dds_transf <- as.data.frame(cell_dds_results) %>%
    bind_cols(feature = rownames(cell_dds_results)) %>%
    mutate(log_padj = - log(.data$padj, base = 10))

## ----dgeCounts, echo=FALSE----------------------------------------------------
dge_cells <-
    filter(
        .data = cell_dds_transf,
        abs(.data$log2FoldChange) > 1, .data$padj < 0.05
    )
dge_subj <-
    filter(
        .data = subj_dds_transf,
        abs(.data$log2FoldChange) > 1, .data$padj < 0.05
    )

## ----plotVolcano, fig.cap=cap_volcano-----------------------------------------
# Function to add theme for ggplots of DESeq2 results.
deseq_themes <- function() {
    list(
        theme_classic(),
        lims(x = c(-4, 5), y = c(0, 80)),
        labs(
            x = "log<sub>2</sub> (fold change)",
            y = "-log<sub>10</sub> (p<sub>adj</sub>)"
        ),
        ggplot2::theme(
            axis.title.x = ggtext::element_markdown(),
            axis.title.y = ggtext::element_markdown())
    )
}

# Build ggplots to visualize subject-level differential expression in scRNA-seq
ggplot_full <- ggplot(data = cell_dds_transf) +
    geom_point(aes(x = log2FoldChange, y = log_padj), na.rm = TRUE) +
    geom_point(
        data = filter(
            .data = cell_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange, y = log_padj), color = "red"
    ) +
    deseq_themes()

ggplot_subj <- ggplot(data = subj_dds_transf) +
    geom_point(aes(x = log2FoldChange, y = log_padj), na.rm = TRUE) +
    geom_point(
        data = filter(
            .data = subj_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange, y = log_padj), color = "red"
    ) +
    geom_label(
        data = filter(
            .data = subj_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange + 0.5, y = log_padj + 5, label = feature)
    ) +
    deseq_themes()

cowplot::plot_grid(ggplot_full, ggplot_subj, ncol = 2, labels = c("A", "B"))

## ----plotGeneCounts, fig.cap=cap_counts---------------------------------------
# Extract counts subset by gene to plot normalized counts.
ggplot_counts <- function(dds_obj, gene) {
    norm_counts <-
        counts(dds_obj, normalized = TRUE)[grepl(gene, rownames(dds_obj)), ]
    sc_counts <-
        data.frame(
            norm_count = norm_counts,
            subject = colData(dds_obj)[["orig.ident"]],
            genotype = factor(
                colData(dds_obj)[["Genotype"]],
                levels = c("WT", "CFTRKO")
            )
        )

    count_ggplot <- ggplot(data = sc_counts) +
        geom_jitter(
            aes(x = genotype, y = norm_count, color = genotype),
            height = 0, width = 0.05
        ) +
        scale_color_manual(
            "Genotype", values = c("WT" = "blue", "CFTRKO" = "red")
        ) +
        lims(x = c("WT", "CFTRKO"), y = c(0, 350)) +
        labs(x = "Genotype", y = "Normalized Counts") +
        ggtitle(label = gene) +
        theme_classic()
    return(count_ggplot)
}

cowplot::plot_grid(
    ggplot_counts(dds_obj = subj_dds, gene = "CFTR") +
        theme(legend.position = "FALSE"),
    ggplot_counts(dds_obj = subj_dds, gene = "CD36") +
        theme(legend.position = "FALSE"),
    cowplot::get_legend(
        plot = ggplot_counts(dds_obj = subj_dds, gene = "CD36")
    ),
    ncol = 3, rel_widths = c(4, 4, 1)
)

## -----------------------------------------------------------------------------
sessionInfo()

#' Small Airway `SingleCellExperiment` object
#'
#' Small airway epithelium single-cell RNA sequencing data subset combined from
#' 7 porcine individuals. Genotypes represent wildtype (Genotype `WT`; n=3)
#' and CFTR-knockout subjects (Genotype `CFTRKO`; n=4) expressing a
#' cystic fibrosis phenotype.\cr
#' \cr
#' To access cell counts and column metadata:
#' \itemize{
#'   \item counts: `SummarizedExperiment::assay(small_airway, "counts")`
#'   \item metadata: `SummarizedExperiment::colData(small_airway)`
#'   }
#' Cell types include:
#' \itemize{
#'   \item Secretory cell
#'   \item Endothelial cell
#'   \item Immune cell
#' }
#'
#' @format A \linkS4class{SingleCellExperiment} object with feature counts of
#' 1311 genes (rows) from 2687 individual cells (columns).
#' Features were subset by gene ontology annotation "ion transport" with
#' included child terms ("anion transport", "cation transport", etc.).
#' Includes a count matrix accessed by
#' \link[SummarizedExperiment:SummarizedExperiment-class]{assay} and
#' column metadata accessed by
#' \link[SummarizedExperiment:SummarizedExperiment-class]{colData}.
#' Metadata variables are defined as:
#' \describe{
#'   \item{orig.ident}{Biological sample identifier (i.e. subject)}
#'   \item{nCount_RNA}{Number of UMIs (i.e. depth/size factor)}
#'   \item{nFeature_RNA}{Features with non-zero counts
#'     (i.e. identified features)}
#'   \item{Genotype}{Sample Genotype ('CF' == cystic fibrosis)
#'     \itemize{
#'       \item WT = non-CF pig
#'       \item CFTRKO = CFTR knockout}
#'     }
#'   \item{Sample}{Airway region of sample}
#'   \item{celltype}{Cell type labels}
#' }
#'
"small_airway"

#' SingleCellExperiment subject values
#'
#' Extract unique values from \linkS4class{SingleCellExperiment}
#' column (i.e. cell) metadata. Used to determine subject and cell type values.
#'
#' @param ... Named metadata variables for subjects and cell types.
#' @inheritParams aggregateBioVar
#' @export
#'
#' @return List of character vectors with unique values from
#' \linkS4class{SingleCellExperiment} column metadata variables.
#'
#' @examples
#' ## Examples of metadata column variable names.
#' names(SummarizedExperiment::colData(small_airway))
#'
#' ## Return list of subject and cell type values from experiment metadata.
#' scSubjects(scExp=small_airway, subjects="orig.ident", cellTypes="celltype")
#'
scSubjects <- function(scExp, ...) {
    metavars <- list(...)
    stopifnot(!is.null(names(metavars)))
    metavar_names <- vapply(
        X=names(metavars), FUN.VALUE=logical(1),
        FUN=function(metavar_name) nchar(metavar_name) > 0
    )
    if (FALSE %in% metavar_names) {
        stop("Names required for: ", metavars[which(metavar_names == FALSE)])
    }
    meta_values <- lapply(metavars, function(var) {
        unique(as.character(scExp[[var]]))
    })
    names(meta_values) <- names(metavars)
    return(meta_values)
}

#' Gene-by-subject count matrix
#'
#' Convert gene-by-cell count matrix to gene-by-subject count matrix.
#' Row sums are calculated for each feature (i.e. gene) across cells by subject.
#'
#' @inheritParams aggregateBioVar
#' @importFrom rlang !! :=
#' @export
#'
#' @return S4 DataFrame of gene-by-subject count sums.
#' @seealso \code{\link{scSubjects}} for `subjects` values.
#'
#' @examples
#' ## Return cell count matrix aggregated by subject.
#' countsBySubject(scExp=small_airway, subjectVar="orig.ident")
#'
countsBySubject <- function(scExp, subjectVar) {

    subjects <- scSubjects(scExp=scExp, subjects=subjectVar)[["subjects"]]

    subjectCounts <-
        lapply(subjects, function(subjectID) {
            subjectMatches <- which(scExp[[subjectVar]] == subjectID)
            if (length(subjectMatches) == 1) {
                subjectSums <-
                    SingleCellExperiment::counts(scExp)[, subjectMatches]
            } else {
                sce <- SingleCellExperiment::counts(scExp)[, subjectMatches]
                subjectSums <- Matrix::rowSums(x=sce)
            }
            tibble::tibble(!!subjectID := unname(subjectSums))
        })

    subjectCounts <- S4Vectors::DataFrame(subjectCounts)
    rownames(subjectCounts) <- rownames(scExp)
    return(subjectCounts)
}

#' Collate metadata variation
#'
#' Identify single cell experiment metadata variables that are identical
#' within subject (e.g. genotype, treatment, cell line, sample preparation).
#' Effectively excludes metadata variables containing between cell variation.
#' Used as the design matrix for differential expression analysis.
#'
#' @inheritParams aggregateBioVar
#' @export
#'
#' @return Tibble data frame of metadata variables without intrasubject
#'   variation from single cell experiment metadata. Rows correspond to
#'   aggregated cells (i.e. subject / biological replicate) and columns to
#'   metadata attribute variables (e.g. genotype, treatment, cell line).
#'
#' @examples
#' ## Return experiment metadata sans intrasubject variation.
#' subjectMetaData(scExp=small_airway, subjectVar="orig.ident")
#'
subjectMetaData <- function(scExp, subjectVar) {

    subjects <- scSubjects(scExp=scExp, subjects=subjectVar)[["subjects"]]
    indexCellBySubject <- match(subjects, scExp[[subjectVar]])

    ## For each metadata variable, calculate a contingency table of variable
    ## levels by subject. Retain variables with a single intrasubject level.
    indexMetaVar <-
        vapply(
            X=SummarizedExperiment::colData(scExp), FUN.VALUE=logical(1),
            FUN=function(metaVar) {
                crosstab <- table(metaVar, scExp[[subjectVar]])
                ifelse(
                    test=TRUE %in% (colSums(crosstab > 0) > 1),
                    yes=FALSE, no=TRUE)
        })

    ## Subset and remove barcode rowname carryover from metadata rownames.
    subjectVariation <-
        SummarizedExperiment::colData(scExp)[indexCellBySubject, indexMetaVar]
    rownames(subjectVariation) <- subjectVariation[[subjectVar]]
    return(subjectVariation)
}

#' Aggregate feature counts and metadata by subject
#'
#' Given an input sparse count matrix and corresponding column metadata,
#' aggregate gene counts by subject level. Metadata variables with only
#' inter-subject variation are retained; any variables with cell-level variation
#' within a subject are dropped (e.g. feature / RNA count by cell).
#'
#' @inheritParams aggregateBioVar
#' @export
#'
#' @return \linkS4class{SummarizedExperiment} object with feature counts
#'   aggregated by subject and summarized inter-subject metadata.
#'
#' @examples
#' ## Construct SummarizedExperiment object with gene-by-subject count matrix
#' ## and column metadata summarized to exclude intrasubject variation.
#' ## See `SummarizedExperiment` accessor functions `assay()` and `colData()`
#' ## to access the count matrix and column metadata for downstream analyses.
#' summarizedCounts(scExp=small_airway, subjectVar="orig.ident")
#'
summarizedCounts <- function(scExp, subjectVar) {
    subjectCounts <- countsBySubject(scExp=scExp, subjectVar=subjectVar)
    summarizedMetadata <- subjectMetaData(scExp=scExp, subjectVar=subjectVar)
    countSummary <-
        SummarizedExperiment::SummarizedExperiment(
            assays=list(counts=subjectCounts),
            colData=summarizedMetadata
        )
    return(countSummary)
}

context("SingleCellExperiment metadata")

test_that("Unique subject and cell type values", {
    ## `orig.ident` == character vector, `celltype` == factor vector
    assayInfo <-
        scSubjects(
            scExp = small_airway,
            subjects = "orig.ident", cellTypes = "celltype"
        )
    expect_equal(
        assayInfo$subjects, unique(small_airway$orig.ident)
    )
    expect_equal(
        assayInfo$cellTypes, as.character(unique(small_airway$celltype))
    )
    ## stopifnot() error for null names
    expect_error(
        scSubjects(scExp = small_airway, "orig.ident", "celltype")
    )
    ## stop() error for missing name in dot param input
    expect_error(
        scSubjects(scExp = small_airway, "orig.ident", cellTypes = "celltype")
    )

})

test_that("Feature count aggregation by subject", {
    subjectCounts <-
        countsBySubject(scExp = small_airway, subjectVar = "orig.ident")
    expect_s4_class(subjectCounts, "DataFrame")
    expect_identical(rownames(subjectCounts), rownames(small_airway))
    expect_error(
        countsBySubject(scExp = small_airway, "orig.ident", "celltype")
    )

    ## From immune cell subset, assign list of cell indexes by subject.
    ic_cells <- small_airway[, grep(pattern = "Immune cell",
                                    x = as.character(small_airway$celltype))]
    subject_index <-
        lapply(
            X = unique(small_airway$orig.ident),
            FUN = function(subject) {
                grep(pattern = subject, x = ic_cells$orig.ident)
        })

    ## Test for single subject matches to prevent rowSums() dimension error.
    subject_sample <- sample(length(subject_index), size = 2)
    subject_subset <-
        subset(subject_index, ifelse(seq_along(subject_index) %in%
                                        subject_sample, FALSE, TRUE))
    subset_index <- c(
        unlist(subject_subset),
        subject_index[subject_sample][[1]][1],
        subject_index[subject_sample][[2]][1]
    )
    cell_subset <- ic_cells[, subset_index]
    expect_gte(
        length(which(table(cell_subset$orig.ident) %in% 1 == TRUE)), 2
    )
    expect_silent(countsBySubject(cell_subset, subjectVar = "orig.ident"))
})

test_that("Subject metadata summary", {
    summarizedMetadata <-
        subjectMetaData(scExp = small_airway, subjectVar = "orig.ident")
    expect_s4_class(summarizedMetadata, "DataFrame")
    expect_equivalent(
        summarizedMetadata$orig.ident,
        unique(small_airway$orig.ident)
    )
    expect_equivalent(
        unique(summarizedMetadata$Genotype),
        unique(small_airway$Genotype)
    )
    expect_equivalent(
        rownames(summarizedMetadata),
        unique(small_airway[["orig.ident"]])
    )
})

test_that("Aggregate SummarizedExperiment output", {
     summarizedSubjects <-
        summarizedCounts(scExp = small_airway, subjectVar = "orig.ident")
    expect_identical(
        SummarizedExperiment::assay(summarizedSubjects, "counts"),
        countsBySubject(scExp = small_airway, subjectVar = "orig.ident")
    )
    expect_identical(
        SummarizedExperiment::colData(summarizedSubjects),
        subjectMetaData(scExp = small_airway, subjectVar = "orig.ident")
    )
    expect_s4_class(summarizedSubjects, "SummarizedExperiment")
})

---
title: "Multi-subject scRNA-seq Analysis"
author:
- name: Jason Ratcliff
  affiliation: &iihg Iowa Institute of Human Genetics, Roy J. and Lucille A.
    Carver College of Medicine, University of Iowa, Iowa City, IA 52242, USA
- name: Andrew Thurman
  affiliation: &intMed Department of Internal Medicine, Roy J. and Lucille A.
    Carver College of Medicine, University of Iowa, Iowa City, IA 52242, USA
- name: Michael Chimenti
  affiliation: *iihg
- name: Alejandro Pezzulo
  affiliation: *intMed
link-citations: true
output: BiocStyle::html_document
bibliography: references.bib
csl: bioinformatics.csl
package: aggregateBioVar
abstract: |
    A method for incorporating *biological replication* into differential gene
    expression analysis of single cell RNA sequencing data.
vignette: |
    %\VignetteIndexEntry{Multi-subject scRNA-seq Analysis}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
nocite: |
    @R-cowplot
    @R-ggtext
    @R-magrittr
    @R-ggtext
    @R-dplyr
---

```{r vignette, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
knitr::opts_knit$set(
    eval.after = "fig.cap"
)
```

```{r setup, message=FALSE}
# For analysis of scRNAseq data
library(aggregateBioVar)
library(SummarizedExperiment, quietly = TRUE)
library(SingleCellExperiment, quietly = TRUE)
library(DESeq2, quietly = TRUE)

# For data transformation and visualization
library(magrittr, quietly = TRUE)
library(dplyr, quietly = TRUE)
library(ggplot2, quietly = TRUE)
library(cowplot, quietly = TRUE)
library(ggtext, quietly = TRUE)
```

```{r links, echo=FALSE}
link_sce <-
    paste0(
        "https://bioconductor.org/packages/release/bioc/",
        "vignettes/SingleCellExperiment/inst/doc/intro.html"
    )
link_se <-
    paste0(
        "https://bioconductor.org/packages/release/bioc/",
        "vignettes/SummarizedExperiment/inst/doc/SummarizedExperiment.html"
    )
link_ncbi <-
    paste0(
        "https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/",
        "wwwtax.cgi?mode=Info&id=9823"
    )
link_DESeq2 <-
    paste0(
        "https://bioconductor.org/packages/devel/bioc/",
        "vignettes/DESeq2/inst/doc/DESeq2.html"
    )
```

```{r captions, echo=FALSE, message=FALSE}
cap_metadata <-
    SummarizedExperiment::colData(x = small_airway) %>% tibble::as_tibble() %>%
    dplyr::select("orig.ident", "Genotype") %>%
    dplyr::distinct() %>% dplyr::arrange(.data$orig.ident)

# Figure 1
cap_pheatmanp_cell <-
    paste0(
        "Gene-by-cell Count Matrix. ",
        "Heatmap of *secretory cell* gene expression with log~2~ counts ",
        "per million cells. Includes ",
        length(grep("Secretory cell", small_airway$celltype)),
        " cells from ", length(unique(small_airway$orig.ident)), " subjects",
        " with genotypes `WT` (n=", length(grep("WT", cap_metadata$Genotype)),
        ", cells=",
        length(
            intersect(
                grep("WT", small_airway$Genotype),
                grep("Secretory cell", small_airway$celltype)
            )
        ),
        ") and `CFTRKO` (n=", length(grep("CFTRKO", cap_metadata$Genotype)),
        ", cells=",
        length(
            intersect(
                grep("CFTRKO", small_airway$Genotype),
                grep("Secretory cell", small_airway$celltype)
            )
        ),
        ")."
    )

# Figure 2
cap_pheatmanp_subj <-
    paste0(
        "Gene-by-subject Count Matrix. ",
        "Heatmap of gene counts aggregated by ",
        "subject with `aggregateBioVar()`.",
        "The `SummarizedExperiment` object with the *Secretory cells* subset ",
        "contains gene counts summed by subject. Aggregate gene-by-subject ",
        "counts are used as input for bulk RNA-seq tools."
    )

# Figure 3
cap_volcano <-
    paste0(
        "Differential expression analysis of scRNA-seq data. ",
        "Comparison of differential expression in ",
        "secretory cells from small airway epithelium ",
        "without aggregation (A) and after aggregation of gene counts ",
        "by subject (biological replicates; B). ",
        "Genes with an absolute log~2~ fold change greater than 1 and an ",
        "adjusted P value less than 0.05 are highlighted in red. Aggregation ",
        "of counts by subject reduced the number of differentially expressed ",
        "genes to CD36 and CFTR for the secretory cell subset."
    )

# Figure 4
cap_counts <-
    paste0(
        "Normalized within-subject gene counts. ",
        "Gene counts aggregated by subject for significantly differentially ",
        "expressed genes from the secretory cell subset."
    )
```

# Introduction

Single cell RNA sequencing (scRNA-seq) studies allow gene expression
quantification at the level of individual cells, and these studies introduce
multiple layers of biological complexity. These include variations in gene
expression between cell states within a sample
(*e.g.*, T cells versus macrophages), between samples within a population
(*e.g.*, biological or technical replicates), and between populations
(*e.g.*, healthy versus diseased individuals).
Because many early scRNA-seq studies involved analysis of only a single sample,
many bioinformatics tools operate on the first layer, comparing gene expression
between cells within a sample.
This software is aimed at organizing scRNA-seq data to permit analysis in the
latter two layers, comparing gene expression between samples and between
populations. An example is given with an implementation of differential
gene expression analysis between populations. From scRNA-seq data stored as a
[`SingleCellExperiment`](`r link_sce`) [@R-SingleCellExperiment] object
with pre-defined cell states, `aggregateBioVar()` stratifies data as a list of
[`SummarizedExperiment`](`r link_se`) [@R-SummarizedExperiment] objects,
a standard [Bioconductor](https://bioconductor.org) data structure for
downstream analysis of RNA-seq data.

# Case Study: Small Airway Epithelium in Cystic Fibrosis

To illustrate the utility of *biological replication* for scRNA-seq
sequencing experiments, consider a set of single cell data from
**porcine small airway epithelium**. In this study, small airway (< 2 mm) tissue
samples were collected from newborn pigs ([*Sus scrofa*](`r link_ncbi`)) to
investigate gene expression patterns and cellular composition in a cystic
fibrosis phenotype. Single cell sequencing samples were prepared using a 10X
Genomics Chromium controller and sequenced on an Illumina HiSeq4000. Data
obtained from seven individuals include both non-CF
(**CFTR+/+**; genotype `WT`; n=4) and CFTR-knockout subjects expressing a
cystic fibrosis phenotype (**CFTR-/-**; genotype `CFTRKO`; n=3).
Cell types were determined following a standard scRNA-seq pipeline using
[Seurat](https://satijalab.org/seurat/) [@Seurat2019], including cell count
normalization, scaling, determination of highly variable genes,
dimension reduction via principal components analysis, and shared
nearest neighbor clustering. Both unsupervised marker detection
(via `Seurat::FindMarkers()`) and a list of known marker genes were used to
annotate cell types. The full data set has been uploaded to the
[Gene Expression Omnibus](https://www.ncbi.nlm.nih.gov/geo/)
as accession number **GSE150211.**

## The `small_airway` Dataset

A subset of `r nrow(small_airway)` genes from `r ncol(small_airway)` cells
assigned as secretory, endothelial, and immune cell types are available in the
`small_airway` data set.
The data are formatted as a `SingleCellExperiment` class S4 object
[@SingleCellExperiment2020], an extension of the
`RangedSummarizedExperiment` class from the `SummarizedExperiment` package.

```{r singleCellExperiment}
small_airway
```

The primary data in `SingleCellExperiment` objects are stored in the `assays`
slot. Here, a single assay `counts` contains gene counts from the single cell
sequencing data. Each column of the assay count matrix represents a cell and
each row a feature (*e.g.*, gene).
Assay slot data can be obtained by `SummarizedExperiment::assay()`, indicating
the`SingleCellExperiment` object and name of the assay slot (here, `"counts"`).
In the special case of the assay being named `"counts"`, the data can be
accessed with `SingleCellExperiment::counts()`.

```{r assays}
assays(small_airway)

# Dimensions of gene-by-cell count matrix
dim(counts(small_airway))

# Access dgCMatrix with gene counts
counts(small_airway)[1:5, 1:30]
```

```{r subjects, echo=FALSE}
sbj_cf <-
    grep(pattern = "CF", x = unique(small_airway$orig.ident), value = TRUE)
sbj_wt <-
    grep(pattern = "WT", x = unique(small_airway$orig.ident), value = TRUE)
```

## Cell Metadata

`SingleCellExperiment` objects may also include column metadata with
additional information annotating individual cells.
Here, the metadata variable `orig.ident` identifies the biological sample of the
cell while `celltype` indicates the assigned cellular identity.
Of the `r length(unique(small_airway$orig.ident))` individual
subjects, 3 are CF (`r sbj_cf`) and 4 are non-CF (`r sbj_wt`).
`Genotype` indicates the sample genotype, one of `WT` or `CFTRKO` for non-CF and
CF subjects, respectively.
Column metadata from `SingleCellExperiment` objects can be accessed with the `$`
operator, where the length of a metadata column variable is equal to the number
of columns (*i.e.*, cells) in the feature count matrix from the `assays` slot.

```{r metadata}
# Subject values
table(small_airway$orig.ident)

# Cell type values
table(small_airway$celltype)

# Subject genotype
table(small_airway$Genotype)
```

The experiment metadata are included as an S4 `DataFrame` object.
To access the full column metadata from `SingleCellExperiment` objects, use
`colData()` from the `SummarizedExperiment` package.
Here, metadata include
`r ncol(colData(small_airway))` variables for each of
`r nrow(colData(small_airway))` cells.
In addition to the biological sample identifier, cell type, and genotype,
metadata include total unique molecular identifiers (UMIs) and
number of detected features.

```{r columnData}
colData(small_airway)
```

## Aggregating Gene Counts

The main functionality of this package involves two generalizable operations:

1. Aggregate within-subject gene counts
2. Summarize metadata to retain inter-subject variation

A wrapper function `aggregateBioVar()` abstracts away these operations
and applies them on a by-cell type basis. For input, a `SingleCellExperiment`
object containing gene counts should contain metadata variables for the
subject by which to aggregate cells (*e.g.*, biological sample) and the assigned
cell types.

### Gene-by-subject Count Matrix

The first operation involves summing all gene counts by subject.
For each gene, counts from all cells within each subject are combined. A
*gene-by-cell* count matrix is converted into a *gene-by-subject* count matrix.

```{r countsBySubject}
countsBySubject(scExp = small_airway, subjectVar = "orig.ident")
```

### Subject Metadata

The second operation removes metadata variables with intrasubject
variation. This effectively retains *inter-subject metadata* and eliminates
variables with intrasubject (*i.e.*, intercellular) variation
(*e.g.*, feature or gene counts by cell). This summarized metadata is used
for modeling a differential expression design matrix.

```{r subjectMetaData}
subjectMetaData(scExp = small_airway, subjectVar = "orig.ident")
```

### Return `SummarizedExperiment`

Both the gene count aggregation and metadata collation steps are combined in
`summarizedCounts()`. This function returns a `SummarizedExperiment` object with
the gene-by-subject count matrix in the `assays` slot, and the summarized
inter-subject metadata as `colData`. Notice the column names now correspond
to the subject level, replacing the cellular barcodes in the
`SingleCellExperiment` following aggregation of gene counts from
within-subject cells.

```{r summarizedCounts}
summarizedCounts(scExp = small_airway, subjectVar = "orig.ident")
```

## `aggregateBioVar()`

These operations are applied to each cell type subset with `aggregateBioVar()`.
The full `SingleCellExperiment` object is subset by cell type (*e.g.*,
secretory, endothelial, and immune cell), the gene-by-subject aggregate count
matrix and collated metadata are tabulated, and a `SummarizedExperiment`
object for that cell type is constructed. A list of `SummarizedExperiment`
objects output by `summarizedCounts()` is the returned to the user. The first
element contains the aggregate `SummarizedExperiment` across all cells, and
subsequent list elements correspond to the cell type indicated by the metadata
variable `cellVar`:

```{r aggregateBioVar}
aggregateBioVar(scExp = small_airway,
                subjectVar = "orig.ident", cellVar = "celltype")
```

# Application to Differential Gene Expression (DGE)

In this case, we want to test for differential
expression between non-CF and CF pigs in the `Secretory cell` subset.
To do so, `aggregateBioVar()` is run on the `SingleCellExperiment` object
by indicated the metadata variables representing the subject-level
(`subjectVar`) and assigned cell type (`cellVar`).
If multiple assays are included in the input `scExp` object,
the **first assay slot** is used.

```{r DESeq2Example}
# Perform aggregation of counts and metadata by subject and cell type.
aggregate_counts <-
    aggregateBioVar(
        scExp = small_airway,
        subjectVar = "orig.ident", cellVar = "celltype"
    )
```

## Exploratory Data Analysis

To visualize the gene-by-subject count aggregation, consider a function to
calculate log<sub>2</sub> counts per million cells and display a heatmap
of normalized expression using `pheatmap` [@R-pheatmap].
`RColorBrewer` [@R-RColorBrewer] and `viridis` [@R-viridis] are used to
generate discrete and continuous color scales, respectively.

```{r pheatmapFxn}
#' Single-cell Counts `pheatmap`
#'
#' @param sumExp `SummarizedExperiment` or `SingleCellExperiment` object
#'   with individual cell or aggregate counts by-subject.
#' @param logSample Subset of log2 values to include for clustering.
#' @param ... Forwarding arguments to pheatmap
#' @inheritParams aggregateBioVar
#'
scPHeatmap <- function(sumExp, subjectVar, gtVar, logSample = 1:100, ...) {
    orderSumExp <- sumExp[, order(sumExp[[subjectVar]])]
    sumExpCounts <- as.matrix(
        SummarizedExperiment::assay(orderSumExp, "counts")
    )
    logcpm <- log2(
        1e6*t(t(sumExpCounts) / colSums(sumExpCounts)) + 1
    )
    annotations <- data.frame(
        Genotype = orderSumExp[[gtVar]],
        Subject = orderSumExp[[subjectVar]]
    )
    rownames(annotations) <- colnames(orderSumExp)

    singleCellpHeatmap <- pheatmap::pheatmap(
        mat = logcpm[logSample, ], annotation_col = annotations,
        cluster_cols = FALSE, show_rownames = FALSE, show_colnames = FALSE,
        scale = "none", ...
    )
    return(singleCellpHeatmap)
}

```

Without aggregation, bulk RNA-seq methods for differential expression analysis
would be applied at the cell level (here, secretory cells;
Figure \@ref(fig:pheatmapCells)).

```{r pheatmapCells, fig.cap=cap_pheatmanp_cell}
# Subset `SingleCellExperiment` secretory cells.
sumExp <- small_airway[, small_airway$celltype == "Secretory cell"]

# List of annotation color specifications for pheatmap.
ann_colors <- list(
    Genotype = c(CFTRKO = "red", WT = "black"),
    Subject = c(RColorBrewer::brewer.pal(7, "Accent"))
)
ann_names <- unique(sumExp[["orig.ident"]])
names(ann_colors$Subject) <- ann_names[order(ann_names)]

# Heatmap of log2 expression across all cells.
scPHeatmap(
    sumExp = sumExp, logSample = 1:100,
    subjectVar = "orig.ident", gtVar = "Genotype",
    color = viridis::viridis(75), annotation_colors = ann_colors,
    treeheight_row = 0, treeheight_col = 0
)
```

----

Summation of gene counts across all cells creates a "pseudo-bulk" data set
on which a subject-level test of differential expression is applied 
(Figure \@ref(fig:pheatmapSubject)).

```{r pheatmapSubject, fig.cap=cap_pheatmanp_subj}
# List of `SummarizedExperiment` objects with aggregate subject counts.
scExp <-
    aggregateBioVar(
        scExp = small_airway,
        subjectVar = "orig.ident", cellVar = "celltype"
    )

# Heatmap of log2 expression from aggregate gene-by-subject count matrix.
scPHeatmap(
    sumExp = aggregate_counts$`Secretory cell`, logSample = 1:100,
    subjectVar = "orig.ident", gtVar = "Genotype",
    color = viridis::viridis(75), annotation_colors = ann_colors,
    treeheight_row = 0, treeheight_col = 0
)
```

## DGE with DESeq2

To run [`DESeq2`](`r link_DESeq2`) [@DESeq2], a `DESeqDataSet` object can be
constructed using `DESeqDataSetFromMatrix()`.
Here, the aggregate counts and subject metadata from the secretory cell subset
are modeled by the variable `Genotype`.
Differential expression analysis is performed with `DESeq` and a results table
is extracted by `results()` to obtain log~2~ fold changes with p-values and
adjusted p-values.

```{r DESeq2Aggregate}
subj_dds_dataset <-
    DESeqDataSetFromMatrix(
        countData = assay(aggregate_counts$`Secretory cell`, "counts"),
        colData = colData(aggregate_counts$`Secretory cell`),
        design = ~ Genotype
    )

subj_dds <- DESeq(subj_dds_dataset)

subj_dds_results <-
    results(subj_dds, contrast = c("Genotype", "WT", "CFTRKO"))
```

For comparison of differential expression with and without aggregation of
gene-by-subject counts, a subset of all secretory cells is used to construct
a `DESeqDataSet` and analysis of differential expression is repeated.

```{r DESeq2Cells}
cells_secretory <-
    small_airway[, which(
        as.character(small_airway$celltype) == "Secretory cell")]
cells_secretory$Genotype <- as.factor(cells_secretory$Genotype)

cell_dds_dataset <-
    DESeqDataSetFromMatrix(
        countData = assay(cells_secretory, "counts"),
        colData = colData(cells_secretory),
        design = ~ Genotype
    )

cell_dds <- DESeq(cell_dds_dataset)

cell_dds_results <-
    results(cell_dds, contrast = c("Genotype", "WT", "CFTRKO"))
```

Add a new variable with log~10~-transformed adjusted P-values.

```{r logPadj, message=FALSE}
subj_dds_transf <- as.data.frame(subj_dds_results) %>%
    bind_cols(feature = rownames(subj_dds_results)) %>%
    mutate(log_padj = - log(.data$padj, base = 10))

cell_dds_transf <- as.data.frame(cell_dds_results) %>%
    bind_cols(feature = rownames(cell_dds_results)) %>%
    mutate(log_padj = - log(.data$padj, base = 10))
```

## Results

```{r dgeCounts, echo=FALSE}
dge_cells <-
    filter(
        .data = cell_dds_transf,
        abs(.data$log2FoldChange) > 1, .data$padj < 0.05
    )
dge_subj <-
    filter(
        .data = subj_dds_transf,
        abs(.data$log2FoldChange) > 1, .data$padj < 0.05
    )
```

DGE is summarized by volcano plot `ggplot` [@R-ggplot2] to show cell-level
(Figure \@ref(fig:plotVolcano)A)
and subject-level tests (Figure \@ref(fig:plotVolcano)B).
Aggregation of gene counts by subject reduced the number of genes with both
an adjusted p-value < 0.05 and an absolute log~2~ fold change > 1 from
`r nrow(dge_cells)` genes to `r nrow(dge_subj)` (Figure \@ref(fig:plotVolcano)).

```{r plotVolcano, fig.cap=cap_volcano}
# Function to add theme for ggplots of DESeq2 results.
deseq_themes <- function() {
    list(
        theme_classic(),
        lims(x = c(-4, 5), y = c(0, 80)),
        labs(
            x = "log<sub>2</sub> (fold change)",
            y = "-log<sub>10</sub> (p<sub>adj</sub>)"
        ),
        ggplot2::theme(
            axis.title.x = ggtext::element_markdown(),
            axis.title.y = ggtext::element_markdown())
    )
}

# Build ggplots to visualize subject-level differential expression in scRNA-seq
ggplot_full <- ggplot(data = cell_dds_transf) +
    geom_point(aes(x = log2FoldChange, y = log_padj), na.rm = TRUE) +
    geom_point(
        data = filter(
            .data = cell_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange, y = log_padj), color = "red"
    ) +
    deseq_themes()

ggplot_subj <- ggplot(data = subj_dds_transf) +
    geom_point(aes(x = log2FoldChange, y = log_padj), na.rm = TRUE) +
    geom_point(
        data = filter(
            .data = subj_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange, y = log_padj), color = "red"
    ) +
    geom_label(
        data = filter(
            .data = subj_dds_transf,
            abs(.data$log2FoldChange) > 1, .data$padj < 0.05
        ),
        aes(x = log2FoldChange + 0.5, y = log_padj + 5, label = feature)
    ) +
    deseq_themes()

cowplot::plot_grid(ggplot_full, ggplot_subj, ncol = 2, labels = c("A", "B"))
```

----

From the significantly differentially expressed genes CFTR and CD36,
the aggregate counts by subject are plotted in Figure \@ref(fig:plotGeneCounts).

```{r plotGeneCounts, fig.cap=cap_counts}
# Extract counts subset by gene to plot normalized counts.
ggplot_counts <- function(dds_obj, gene) {
    norm_counts <-
        counts(dds_obj, normalized = TRUE)[grepl(gene, rownames(dds_obj)), ]
    sc_counts <-
        data.frame(
            norm_count = norm_counts,
            subject = colData(dds_obj)[["orig.ident"]],
            genotype = factor(
                colData(dds_obj)[["Genotype"]],
                levels = c("WT", "CFTRKO")
            )
        )

    count_ggplot <- ggplot(data = sc_counts) +
        geom_jitter(
            aes(x = genotype, y = norm_count, color = genotype),
            height = 0, width = 0.05
        ) +
        scale_color_manual(
            "Genotype", values = c("WT" = "blue", "CFTRKO" = "red")
        ) +
        lims(x = c("WT", "CFTRKO"), y = c(0, 350)) +
        labs(x = "Genotype", y = "Normalized Counts") +
        ggtitle(label = gene) +
        theme_classic()
    return(count_ggplot)
}

cowplot::plot_grid(
    ggplot_counts(dds_obj = subj_dds, gene = "CFTR") +
        theme(legend.position = "FALSE"),
    ggplot_counts(dds_obj = subj_dds, gene = "CD36") +
        theme(legend.position = "FALSE"),
    cowplot::get_legend(
        plot = ggplot_counts(dds_obj = subj_dds, gene = "CD36")
    ),
    ncol = 3, rel_widths = c(4, 4, 1)
)
```

# References

<div id="refs"></div>

# Session Info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{small_airway}
\alias{small_airway}
\title{Small Airway \code{SingleCellExperiment} object}
\format{
A \linkS4class{SingleCellExperiment} object with feature counts of
1311 genes (rows) from 2687 individual cells (columns).
Features were subset by gene ontology annotation "ion transport" with
included child terms ("anion transport", "cation transport", etc.).
Includes a count matrix accessed by
\link[SummarizedExperiment:SummarizedExperiment-class]{assay} and
column metadata accessed by
\link[SummarizedExperiment:SummarizedExperiment-class]{colData}.
Metadata variables are defined as:
\describe{
\item{orig.ident}{Biological sample identifier (i.e. subject)}
\item{nCount_RNA}{Number of UMIs (i.e. depth/size factor)}
\item{nFeature_RNA}{Features with non-zero counts
(i.e. identified features)}
\item{Genotype}{Sample Genotype ('CF' == cystic fibrosis)
\itemize{
\item WT = non-CF pig
\item CFTRKO = CFTR knockout}
}
\item{Sample}{Airway region of sample}
\item{celltype}{Cell type labels}
}
}
\usage{
small_airway
}
\description{
Small airway epithelium single-cell RNA sequencing data subset combined from
7 porcine individuals. Genotypes represent wildtype (Genotype \code{WT}; n=3)
and CFTR-knockout subjects (Genotype \code{CFTRKO}; n=4) expressing a
cystic fibrosis phenotype.\cr
\cr
To access cell counts and column metadata:
\itemize{
\item counts: \code{SummarizedExperiment::assay(small_airway, "counts")}
\item metadata: \code{SummarizedExperiment::colData(small_airway)}
}
Cell types include:
\itemize{
\item Secretory cell
\item Endothelial cell
\item Immune cell
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize_counts.R
\name{scSubjects}
\alias{scSubjects}
\title{SingleCellExperiment subject values}
\usage{
scSubjects(scExp, ...)
}
\arguments{
\item{scExp}{\linkS4class{SingleCellExperiment} object containing
(at minimum) gene counts and column metadata describing sample identifiers
and cell types.}

\item{...}{Named metadata variables for subjects and cell types.}
}
\value{
List of character vectors with unique values from
\linkS4class{SingleCellExperiment} column metadata variables.
}
\description{
Extract unique values from \linkS4class{SingleCellExperiment}
column (i.e. cell) metadata. Used to determine subject and cell type values.
}
\examples{
## Examples of metadata column variable names.
names(SummarizedExperiment::colData(small_airway))

## Return list of subject and cell type values from experiment metadata.
scSubjects(scExp=small_airway, subjects="orig.ident", cellTypes="celltype")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate_bio_var.R
\name{aggregateBioVar}
\alias{aggregateBioVar}
\title{Aggregate subject-level biological variation}
\usage{
aggregateBioVar(scExp, subjectVar, cellVar)
}
\arguments{
\item{scExp}{\linkS4class{SingleCellExperiment} object containing
(at minimum) gene counts and column metadata describing sample identifiers
and cell types.}

\item{subjectVar}{Metadata column name assigning biological sample
identity to aggregate within-subject feature counts.}

\item{cellVar}{Metadata column name assigning cell type. Used for
aggregating gene-by-subject count matrices by cell type.}
}
\value{
List of \linkS4class{SummarizedExperiment} objects with
gene-by-subject count matrices and variable inter-subject column
metadata across and within cell types.
}
\description{
Given an input gene-by-cell count matrix from a
\linkS4class{SingleCellExperiment} object, sum within-subject gene counts
into an aggregate gene-by-subject count matrix. Column metadata accessed
by \link[SummarizedExperiment:SummarizedExperiment-class]{colData}
are collated by \link{subjectMetaData} to remove variables with inter-cell
intrasubject variation, effectively retaining between-subject variation.
The summary operations are performed across all cell types and within each
cell type. A list of \linkS4class{SummarizedExperiment} objects is returned
each with aggregate gene-by-subject count matrix and inter-subject metadata.
}
\examples{
## Aggregate gene-by-subject count matrix and inter-subject metadata
aggregateBioVar(
    scExp=small_airway,
    subjectVar="orig.ident", cellVar="celltype"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize_counts.R
\name{countsBySubject}
\alias{countsBySubject}
\title{Gene-by-subject count matrix}
\usage{
countsBySubject(scExp, subjectVar)
}
\arguments{
\item{scExp}{\linkS4class{SingleCellExperiment} object containing
(at minimum) gene counts and column metadata describing sample identifiers
and cell types.}

\item{subjectVar}{Metadata column name assigning biological sample
identity to aggregate within-subject feature counts.}
}
\value{
S4 DataFrame of gene-by-subject count sums.
}
\description{
Convert gene-by-cell count matrix to gene-by-subject count matrix.
Row sums are calculated for each feature (i.e. gene) across cells by subject.
}
\examples{
## Return cell count matrix aggregated by subject.
countsBySubject(scExp=small_airway, subjectVar="orig.ident")

}
\seealso{
\code{\link{scSubjects}} for \code{subjects} values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate_bio_var.R
\name{countsByCell}
\alias{countsByCell}
\title{Within cell type gene-by-subject matrices}
\usage{
countsByCell(scExp, subjectVar, cellVar)
}
\arguments{
\item{scExp}{\linkS4class{SingleCellExperiment} object containing
(at minimum) gene counts and column metadata describing sample identifiers
and cell types.}

\item{subjectVar}{Metadata column name assigning biological sample
identity to aggregate within-subject feature counts.}

\item{cellVar}{Metadata column name assigning cell type. Used for
aggregating gene-by-subject count matrices by cell type.}
}
\value{
List of gene-by-subject and design matrices for each cell type.
}
\description{
Given a vector of unique cell types, calculate a gene-by-subject matrix and
inter-subject metadata for a differential expression design matrix.
}
\examples{
## Return list of `SummarizedExperiments` with gene-by-subject count matrices
## and subject metadata for each unique `SingleCellExperiment` cell type.
countsByCell(
    scExp=small_airway,
    subjectVar="orig.ident", cellVar="celltype"
)

}
\seealso{
\code{\link{summarizedCounts}} for aggregate counts and metadata
\linkS4class{SummarizedExperiment} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize_counts.R
\name{summarizedCounts}
\alias{summarizedCounts}
\title{Aggregate feature counts and metadata by subject}
\usage{
summarizedCounts(scExp, subjectVar)
}
\arguments{
\item{scExp}{\linkS4class{SingleCellExperiment} object containing
(at minimum) gene counts and column metadata describing sample identifiers
and cell types.}

\item{subjectVar}{Metadata column name assigning biological sample
identity to aggregate within-subject feature counts.}
}
\value{
\linkS4class{SummarizedExperiment} object with feature counts
aggregated by subject and summarized inter-subject metadata.
}
\description{
Given an input sparse count matrix and corresponding column metadata,
aggregate gene counts by subject level. Metadata variables with only
inter-subject variation are retained; any variables with cell-level variation
within a subject are dropped (e.g. feature / RNA count by cell).
}
\examples{
## Construct SummarizedExperiment object with gene-by-subject count matrix
## and column metadata summarized to exclude intrasubject variation.
## See `SummarizedExperiment` accessor functions `assay()` and `colData()`
## to access the count matrix and column metadata for downstream analyses.
summarizedCounts(scExp=small_airway, subjectVar="orig.ident")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize_counts.R
\name{subjectMetaData}
\alias{subjectMetaData}
\title{Collate metadata variation}
\usage{
subjectMetaData(scExp, subjectVar)
}
\arguments{
\item{scExp}{\linkS4class{SingleCellExperiment} object containing
(at minimum) gene counts and column metadata describing sample identifiers
and cell types.}

\item{subjectVar}{Metadata column name assigning biological sample
identity to aggregate within-subject feature counts.}
}
\value{
Tibble data frame of metadata variables without intrasubject
variation from single cell experiment metadata. Rows correspond to
aggregated cells (i.e. subject / biological replicate) and columns to
metadata attribute variables (e.g. genotype, treatment, cell line).
}
\description{
Identify single cell experiment metadata variables that are identical
within subject (e.g. genotype, treatment, cell line, sample preparation).
Effectively excludes metadata variables containing between cell variation.
Used as the design matrix for differential expression analysis.
}
\examples{
## Return experiment metadata sans intrasubject variation.
subjectMetaData(scExp=small_airway, subjectVar="orig.ident")

}
